#!/pro/bin/perl

use strict;
use warnings;
use integer;
use DB_File;
use Getopt::Long qw(:config bundling noignorecase);

my ($list, %db, $id, $age) = (4);
for (qw(name ptsitrc country age)) {
    tie %{$db{$_}}, "DB_File", "db/$_";
    }

my %hats = (
    Bold	=> [     0 =>    249, qw( B - )],
    Pro		=> [   250 =>    499, qw( C - )],
    Graduate	=> [   500 =>    999, qw( G M )],
    Wizard	=> [  1000 =>   2499, qw( W - )],
    Royalty	=> [  2500 =>   7499, qw( R K )],
    Pharaoh	=> [  7500 =>  19999, qw( P - )],
    Olympian	=> [ 20000 =>  39999, qw( O - )],
    OlymStar	=> [ 40000 =>  79999, qw( S - )],
    OlymStr2	=> [ 80000 => 999999, qw( D - )],
    );

my $opt_H;
GetOptions (
    "i"   => sub { $db{age}{$_}++ for keys %{$db{age}} },
    "l"   => sub { $list = 1 },	# plain List
    "b"   => sub { $list = 2 },	# plain Barchart
    "h"   => sub { $list = 3 },	# barchart per Hat/Age
    "a"   => sub { $list = 4 },	# barchart per Age/Hat
    "H:s" => \$opt_H,		# Filter on Hat (B, C, G, W, R, P, O, S, D)
    ) or die;
defined $opt_H and $opt_H ||= "P";
if ($opt_H) {
    foreach my $hat (keys %hats) {
	$hats{$hat}[2] eq uc $opt_H ||
	$hats{$hat}[3] eq uc $opt_H and $opt_H = $hat;
	}
    if (exists $hats{$opt_H}) {
	print STDERR "Filtering on hat $opt_H\n";
	}
    else {
	undef $opt_H;
	}
    }

while (print STDERR "ID : ") {
    chomp ($id = uc scalar <STDIN>);
    $id =~ s/^-/CA/;
    $id =~ s/^\+/BR/;
    $id =~ m/^\w+/ or last;
    ($id, $age) = split m/\W+/, $id, 2;
    unless (exists $db{ptsitrc}{$id}) {
	print STDERR "No points for $id\nStill add? [y/N] > N\b";
	scalar <STDIN> =~ m/^[JjYy+]/ or next;
	$db{ptsitrc}{$id} = 0;
	}
    unless ($age) {
	print "age: ";
	chomp ($age = <STDIN>);
	}
    if (exists $db{age}{$id}) {
	print STDERR "$id\'s age was already known ($db{age}{$id})\n";
	$age == $db{age}{$id} or
	    warn "Changing age for $id from $db{age}{$id} to $age\n";
	}
    $age and $db{age}{$id} = $age + 0;
    }

my %age_db = %{$db{age}};
untie %{$db{age}};
delete $age_db{CA76779};

if ($list == 1) {
    foreach $id (sort { $age_db{$a} <=> $age_db{$b} } keys %age_db) {
	my $p = $db{ptsitrc}{$id} // $db{points}{$id} // 0;
	if ($opt_H) {
	    $p >= $hats{$opt_H}[0] && $p <= $hats{$opt_H}[1] or next;
	    }
	printf "%2d %-12s %6d %-20.20s %s\n",
	    $age_db{$id}, $id, $p, $db{name}{$id}, $db{country}{$id};
	}
    exit;
    }

if ($list == 2) {
    my %age;
    $age{$_}++ for values %age_db;
    my %hist;
    $hist{$_ / 10} += $age{$_} for keys %age;
    printf "%d0 - %d9 %3d\n", $_, $_, $hist{$_} for sort { $a <=> $b } keys %hist;
    exit;
    }

if ($list == 3) {
    foreach my $hat (sort { $hats{$a}[0] <=> $hats{$b}[0] } keys %hats) {
	my ($low, $upper) = @{$hats{$hat}};
	$upper == 999999 and $upper = "";
	print "$hat $low .. $upper\n";
	my %age;
	foreach my $id (keys %age_db) {
	    my $pts = $db{ptsitrc}{$id} // $db{points}{$id} // 0;
	    $pts >= $low && $pts <= $hats{$hat}[1] or next;
	    #printf "    %2d %-12s %-20.20s %s\n", $age_db{$id}, $id, $db{name}{$id}, $db{country}{$id};
	    $age{$age_db{$id}}++
	    }
	my %hist;
	$hist{$_ / 10} += $age{$_} for keys %age;
	printf "  %d0 - %d9 %3d\n", $_, $_, $hist{$_} for sort { $a <=> $b } keys %hist;
	}
    exit;
    }

my (@cnt, $age_range);

format XLS_TOP =
+---------+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+
| Age     | Bld | Pro | Gra | Wiz | Roy | Pha | Oly | O*1 | O*2 | sum |
+---------+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+
.
format XLS =
| @>>>>>> | @>> | @>> | @>> | @>> | @>> | @>> | @>> | @>> | @>> | @>> |
$age_range, @cnt
+---------+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+
.

if ($list == 4) {
    my %hist;
    my $gran = 5;
    foreach my $id (keys %age_db) {
	my $pts = $db{ptsitrc}{$id} // $db{points}{$id} // 0;
	foreach my $hat (sort { $hats{$a}[0] <=> $hats{$b}[0] } keys %hats) {
	    $pts >= $hats{$hat}[0] && $pts <= $hats{$hat}[1] or next;
	    $hist{$age_db{$id} / $gran}{$hat}++;
	    $hist{$age_db{$id} / $gran}{sum}++;
	    last;
	    }
	}
    local $~ = "XLS";
    local $^ = "XLS_TOP";
    for (sort { $a <=> $b } keys %hist) {
	$age_range = sprintf "%02d - %02d", $gran * $_, $gran * $_ + $gran - 1;
	@cnt = ();
	foreach my $hat (sort { $hats{$a}[0] <=> $hats{$b}[0] } keys %hats) {
	    $hat eq "sum" and next;
	    push @cnt, exists $hist{$_}{$hat} ? $hist{$_}{$hat} : "-";
	    }
	push @cnt, $hist{$_}{sum};
	{ no warnings;
	  foreach my $i (0..$#cnt) { $hats{sum}[$i] += $cnt[$i] }}
	write;
	}
    $age_range = ""; @cnt = @{$hats{sum}}; delete $hats{sum};
    write;
    }
