#!/pro/bin/perl

# graph.pl [ --top=27 ] [ --window=17 ] [ --width=### ] [ --height=### ]
#          [ id [ pos..pos ]]

use strict;
use warnings;

use File::Copy;
use X11::Protocol;
use DB_File;
use Tk;
use Tk::WinPhoto;

use Getopt::Long qw(:config nopermute bundling noignorecase);
my $opt_w = 17;	# Window
my $x11 = X11::Protocol->new ();
$x11->choose_screen (0); # Root window
my ($cx, $cy, $opt_t) = map { int (.90 * $_) # 80% fill of the full screen
    } $x11->{width_in_pixels}, $x11->{height_in_pixels}; # Graph size
$cx > 1600 and $cx = int (.95 * 1600);	# prevent the use of dual screen
$cx -= 200;	# legenda
GetOptions (
    "t|top:27"     => \$opt_t,
    "w|window=i"   => \$opt_w,
    "x|W|width=i"  => \$cx,
    "y|H|height=i" => \$cy,
    ) or die;
$opt_t      and $opt_w = $opt_t;
$opt_w <  3 and die "Window >= 3 please\n";
$opt_w > 27 and die "Window <= 27 please\n";
$opt_w = int ($opt_w / 2);

my %pts;
tie my %p, "DB_File", "db/ptsitrc";
tie my %n, "DB_File", "db/name";
tie my %q, "DB_File", "db/quote";
tie my %c, "DB_File", "db/country";
foreach my $id (keys %p) {
    #$p{$id} < 5000 and next;
    push @{$pts{$p{$id}}}, $id;
    }

my @p = sort { $b <=> $a } keys %pts;
my %pos;
my ($pos, $x) = (1, 1);
foreach my $pts (@p) {
    foreach my $id (@{$pts{$pts}}) {
	$pos{$id} = [ $pos, $x ];
	$pos{$x++} = $id;
	}
    $pos = $x;
    }
my $title = "ITRC Graph";
my $hl = $opt_t ? $pos{$opt_w + 1} : "BR41967"; # The ID being congratulated
@ARGV and $ARGV[0] eq "me" and $ARGV[0] = "BR41967";
if (@ARGV and exists $p{$ARGV[0]}) {
    $hl = $ARGV[0];
    $title  = "$hl $n{$hl}";
    $title .= " - $c{$hl}" if $c{$hl};
    $title .= " $q{$hl}" if $q{$hl};
    $title =~ s/&quot;/'/g;
    $title =~ s/&nbsp;/ /g;
    $title =~ s/&amp;/&/g;
    $title =~ s/\s+/ /g;
    print STDERR "Pos $hl - $p{$hl} = $pos{$hl}[1]\n";
    my ($mn, $mx) = ($pos{$hl}[1] - $opt_w, $pos{$hl}[1] + $opt_w);
    $mn <  1 and $mn =  1;
    $mx < 20 and $mx = 20;
    my ($id_min, $id_max) = ($pos{$mn}, $pos{$mx});
    # Won't have to check for upper limit in congrats: there will
    # always be at least 50 members towards 0 points
    @ARGV > 1 && $ARGV[1] =~ m/^(\d+)(?:-|\.\.)(\d+)$/ and
	($id_min, $id_max) = ($pos{$1}, $pos{$2});
    @p = @pos{$pos{$id_min}[1] .. $pos{$id_max}[1]};
    @p > 27 and $#p = 26;
    }
else {
    $title = "HP ITRC Top 27";
    @p = @pos{1..27};
    }
my ($y_min, $y_max) = ($p{$p[-1]}, $p{$p[0]});
$y_max = 100 * (int (($y_max + 99) / 100));
print STDERR "$y_max [ $p[0] ($p{$p[0]}) .. $p[-1] ($p{$p[-1]}) ] $y_min .. ";

my @id = @p;

my %g;
my @dt = localtime;
my $today = ((1900 + $dt[5]) * 100 + $dt[4] + 1) * 100 + $dt[3];
foreach my $id (@id) {
    for (`id-grep $id`) {
	s/^DB\b/$today/o;
	m/^(\d{8})\s+\d+!?\s+(\d+)/ or next;
	$g{$1}{$id} = sprintf "%9d", $2;
	$2 < $y_min and $y_min = $2;
	}
    }
$y_min -= $y_min % 100;
$y_min = 0;	# Otherwise the dates don't show
#print STDERR "$y_min\n";

@dt = sort { $a <=> $b } keys %g;

my $w = MainWindow->new;
$w->title ($title);
my $c = $w->Canvas (
    -background		=> "Gray75", # $w->cget (-background),
    -width		=> $cx + 200,
    -height		=> $cy,
    -relief		=> "flat",
    -borderwidth	=>   0,
    -highlightthickness => 0)->pack (-expand => 1);

my $tickLineColor = "Gray90";
my $scoreColor    = "Blue4";
my $dateColor     = "Green4";
my $date_dens = 10;	# Show a date every n tick marks. Too low gets crowded
my $x_ticks   = 100;
my $y_ticks   =  20;
my $y_dif     = $y_max - $y_min;
my $y_int     = $y_dif / $y_ticks;
my ($fx, $fy) = ($cx / @dt, $cy / $y_dif);

foreach my $y (1 .. ($y_ticks - 1)) {
    my $py = $cy - $y * ($cy / $y_ticks);
    $c->createLine (0, $py, $cx, $py,
	-tags	=> "tick-y",
	-fill	=> $tickLineColor,
	-width	=> 1);
    my $txt = $y_min + int ($y * $y_int);
    $c->createText (3, $py - 4,
	-text       => $txt,
	-tags       => "tick-y",
	-fill       => $scoreColor,
	-anchor     => "sw");
    }
$c->update;

#for (my ($i, $x) = (-1, $cx / $grid); $x < $cx; $x += $cx / $grid) {
foreach my $i (0 .. $x_ticks) {
    my $x = $cx * $i / $x_ticks;
    $c->createLine ($x, 0, $x, $cy,
	-tags	=> "tick-x",
	-fill	=> $tickLineColor,
	-width	=> 1);
    $c->update;

    my $d = int ($#dt * $i / $#dt);
    $dt[$d] or next;
    $i % $date_dens == 1 or next;	# Too crowded
    $c->createText ($x, $cy - 5,
	-text   => $dt[$d],
	-tags   => "tick-t",	# Time
	-fill   => $dateColor,
	-anchor => "s");
    }
$c->update;

my %hat = (
      250 => [ "#b0ffb0", "Pro"		],
      500 => [ "#9090ff", "Graduate"	],
     1000 => [ "#b000b0", "Wizard"	],
     2500 => [ "#daaa00", "Royalty"	],
     7500 => [ "#0080b0", "Pharaoh"	],
    20000 => [ "#008000", "Olympian"	],
    40000 => [ "#00b000", "Olympian *"	],
    );
foreach my $hat (keys %hat) {
    $hat >= $y_min && $hat <= $y_max or next;

    my $py = $cy - ($hat - $y_min) * $fy;
    $c->createLine (0, $py, $cx, $py,
	-tags	=> "hat",
	-fill	=> $hat{$hat}[0],
	-width	=> 1);
    $c->createText (50, $py,
	-text   => $hat{$hat}[1],
	-font   => "-misc-fixed-bold-r-normal--13-*-75-75-*-iso8859-1",
	-tags   => "hat",
	-fill   => $hat{$hat}[0],
	-anchor => "sw");
    }
$c->update;

my @clr = map { s/2/ff/g; s/1/7f/g; s/0/00/g; "#$_" } map { "$_" }
    qw( 001 002 102 101 100 200 220 110 010
	210 222 011 012 221 211 201 202 021
	121 111 000 112 122 022 120 020 212
	);
#push @clr, splice @clr, 10, 1;

foreach my $i (reverse (0 .. $#id)) {
    my ($id, @ln) = ($id[$i]);
    foreach my $d (0 .. $#dt) {
	push @ln, $d * $fx, $cy - (($g{$dt[$d]}{$id} // 0) - $y_min) * $fy;
	}
    $g{$dt[-1]}{$id} && $p{$id} > $g{$dt[-1]}{$id} and
	push @ln, $ln[-2] + 10, $cy - ($p{$id} - $y_min) * $fy;

    $c->createLine (@ln,
	-smooth => 1,
	-tags   => $id,
	-fill   => $clr[$i],
	-width  => $id eq $hl ? 3 : 1);

    my $yx = @id < 100 ? 30 : 20;
    $c->createText ($cx + 5, 20 + $i * $yx,
	-text   => sprintf ("%4d %-8s - %5d", $pos{$id}[0], $id, $p{$id}),
	-font   => $id eq $hl ?
	           "-misc-fixed-bold-r-normal--13-*-75-75-*-iso8859-1" :
	           "-misc-fixed-medium-r-normal--13-*-75-75-*-iso8859-1",
	-tags   => $id,
	-fill   => $clr[$i],
	-anchor => "w");

    unless ($yx == 20) {
	$c->createText ($cx + 41, 32 + $i * $yx,
	    -text   => $n{$id},
	    -font   => "-misc-fixed-medium-r-normal--11-*-75-75-*-iso8859-1",
	    -tags   => $id,
	    -fill   => $clr[$i],
	    -anchor => "w");
	}

    $c->update;
    }
$c->createText (60, 20,
    -text   => $title,
    -fill   => "Black",
    -font   => "-misc-fixed-bold-r-normal--15-*-75-75-*-iso8859-1",
    -anchor => "w");

$c->Tk::bind ("<Button-3>" => sub {
    if ($^O eq "includes_the_decoration") {
	my $geo = $w->geometry;
	if ($^O ne "linux") {
	    # WM decoration should be added
	    my @geo = split m/\D/, $geo;
	    $geo[0] += 12; $geo[1] += 32;
	    $geo = "$geo[0]x$geo[1]+$geo[2]+$geo[3]";
	    }
	system "import -window root -crop $geo web/$hl.jpg";
	}
    else {
	my $img = $w->Photo (-format => "Window", -data => oct ($w->id));
	$img->write ("$hl.xpm", -format => "xpm");
	system "convert $hl.xpm web/$hl.jpg";
	unlink "$hl.xpm";
	$w->bell;
	}
    });

MainLoop;
