#!/pro/bin/perl

use strict;
use warnings;

use Tk;
use Tk::WinPhoto;
use Date::Calc qw(Delta_DHMS);
use File::Copy;

use Getopt::Long qw(:config nopermute bundling);

my $log = shift || "speed.log";

my $cx =    1000;	# Canvas width  in dots
my $cy =     300;	# Canvas height in dots

my $xm = 24 * 60;	# Log show period in minutes
my $yc =  150000;	# Upper graph limit of page-1 size
my $yr =      60;	# Upper graph limit of load speed in kbps

my $w;			# The window
my $c;

my $TOD =      0;	# To convert duration to time of day

sub parse
{
    my ($x, $y);

    if (-M $log > 0) {
	$w->after (2000, \&parse);
	return;
	}

    $c->delete ("now");
    $c->delete ("lst");

#   print STDERR "===================\n";
    open LOG, "< $log";
    my $line = <LOG>;
    my ($today) = ($line =~ m/^([-\d]+)/);
    close LOG;
    $c->createText ($cx / 2, 0,
	-text		=> "ITRC read speed monitor started $today",
	-tags		=> "now",
	-fill		=> "Green4",
	-justify	=> "center",
	-anchor		=> "n");

    open LOG, "< $log";

    my $lst = 0;

    my @d = map { $_ + 0 } ($line =~ m/^(\d+)-(\d+)-(\d+)\s+(\d+):(\d+):(\d+)/);
    @d[3,4,5] = (0, 0, 0);
    $TOD = 60 * $d[3] + $d[4];
    my $start = ($d[3] * 60 + $d[4]) * 60 + $d[5];

    my %lst;
    while (<LOG>) {
#	print STDERR;
	# 2003-03-06 09:50:00 107536   3.02 s  34.72 kbps
	my ($date, $time, $size, $duration, $speed) =
	    (m/^([\d-]+)\s+([\d:]+)\s+(\d+)\s+([\d.]+)\s+s\s+([\d.]+)/);

#	print STDERR "\t$time $size $speed\n";
	unless (defined $size) {
	    eof LOG and last;
	    next;
	    }

	my @elapsed = Delta_DHMS (@d,
	    map { $_ + 0 } (m/^(\d+)-(\d+)-(\d+)\s+(\d+):(\d+):(\d+)/));
	my $now = ($elapsed[1] * 60 + $elapsed[2]) * 60 + $elapsed[3] - $start;
	my $m = $now / 60;
	$x = $cx * $m / $xm;

	$lst = $speed;
	rate_ticks ($lst) and last;
	push @{$lst{$date}}, ($x, $cy - $cy * $lst / $yr);
	}
    eof LOG and $^T = time;
    close LOG;

    if (@{$lst{$today}} > 2) {
	# OrangeRed = 255.69.0 (#FF4500)
	my $color = [ 255, 69, 0 ];
	my @lst;

	my @dates = sort keys %lst;
	my $n = @dates;
	@lst = @{$lst{$today = pop @dates}};
	for (@dates) {
	    my $clr = sprintf ("#%02x%02x%02x", 
		map { int ($_ * (100 - (80 * log ($n + 2) / 3.5)) / 100) } @$color);
	    $n--;
	    $c->createLine (@{$lst{$_}},
		-smooth => 1,
		-tags   => "lst",
		-fill   => $clr,
		-width  => 1);
	    }

	$c->createLine (@lst,
	    -smooth => 1,
	    -tags   => "lst",
	    -fill   => sprintf ("#%02x%02x%02x", @$color),
	    -width  => 2);
	my $yy = pop @lst;
	$c->createText ($lst[-1] - 80, .45 * $cy,
	    -text   => "$today : $lst",
	    -tags   => "lst",
	    -fill   => sprintf ("#%02x%02x%02x", @$color),
	    -anchor => "w");
	}
    $w->after (100, \&parse);
    } # parse

$w = MainWindow->new;
$c = $w->Canvas (
    -background		=> shift || $w->cget (-background),
    -width		=> $cx,
    -height		=> $cy,
    -relief		=> "flat",
    -borderwidth	=>   0,
    -highlightthickness	=>   0)->pack (-expand => 1);
chomp (my $model = `model`);
$w->Entry (
    -background		=> $c->cget (-background),
    -relief		=> "flat",
    -justify		=> "center",
    -borderwidth	=>   1,
    -highlightthickness	=>   0)->pack (
	-anchor	=> "center",
	-expand	=> 1,
	-pady	=> ".25c",
	-fill	=> "x")->insert (
	    0, "HP $model");

my $grid = 12;
for (my $x = $cx / $grid; $x < $cx; $x += $cx / $grid) {
    $c->createLine ($x, 0, $x, $cy,
	-tags	=> "tick-x",
	-fill	=> "Gray70",
	-width	=> 1);
    }

my $t = "02";
for (my $x = $cx / $grid; $x < $cx; $x += $cx / $grid) {
    $c->createText ($x, $cy,
	-text   => "$t:00",
	-tags   => "tick-t",	# Time
	-fill   => "Green4",
	-anchor => "s");
    $t++;$t++;
    }

for (my $y = $cy / 6; $y < $cy; $y += $cy /  6) {
    $c->createLine (0, $y, $cx, $y,
	-tags	=> "tick-y",
	-fill	=> "Gray70",
	-width	=> 1);
    }

sub rate_ticks
{
    my $r = shift;

    $r < $yr && return 0;

    $yr += 10 while $r >= $yr;

#   print STDERR "Rate Ticks set to $yr\n";
    $c->delete ("tick-r");
    shw_rate_ticks ();
    } # rate_ticks

sub shw_rate_ticks
{
    for (my $y = 0; $y <= $cy; $y += $cy /  6) {
	my $t = sprintf "%.1f", $yr - ($cy - $y) * ($yr / $cy);
	$c->createText ($cx, $cy - $y + $cy / 30,
	    -text   => "$t ",
	    -tags   => "tick-r",	# Rate
	    -fill   => "Red4",
	    -anchor => "e");
	}
    1;
    } # rate_ticks

shw_rate_ticks ();

$^T = 0;
parse ();

sub make_png ()
{
    $w->update;
#   my $geo = $w->geometry;
#   if ($^O ne "linux") {
#	# WM decoration should be added
#	my @geo = split m/\D/, $geo;
#	$geo[0] += 12; $geo[1] += 32;
#	$geo = "$geo[0]x$geo[1]+$geo[2]+$geo[3]";
#	}
#   system "import -window root -crop $geo web/$log.png";

    my $img = $w->Photo (-format => "Window", -data => oct ($w->id));
    $img->write ("speed.xpm", -format => "xpm");
    system "convert speed.xpm web/$log.png";
    unlink "speed.xpm";
    $w->bell;
    } # make_png

$c->Tk::bind ("<Button-3>" => \&make_png);
make_png;

MainLoop;
