#!/pro/bin/perl

use strict;
use warnings;

use Time::HiRes qw( gettimeofday tv_interval usleep );
use LWP::Simple;

my $opt_v   = $::opt_v + 1 //  1;
my $timeout = $::timeout   // 90;
my $retries = 9;

# http://forums1.itrc.hp.com/service/forums/publicProfile.do?userId=BR691259&forumId=1
my $server  = "forums.itrc.hp.com";
my $servip  = join "." => unpack "CCCC", scalar gethostbyname $server;
my $ux_url  = "service/forums";

sub get_request ($);
sub get_request ($)
{
    my ($url) = @_;

    $opt_v > 7 and print STDERR "Request for '$url'\n";
    my $t0 = [ gettimeofday ];
    my $page;
    for my $try (1 .. $retries) {
	$page = get ("http://$servip$url") and last;
	$opt_v > 7 and print STDERR " .. $try\b\b\b\b\b";
	usleep ($try * 500000);
	}
    if ($page) {
	my $elapsed = tv_interval ($t0);
	my $t = length $page;
	$opt_v and
	    printf STDERR "[ %6d %5.1f kbps ]\n", $t, $t / $elapsed / 1024;
	$opt_v > 9 and
	    print STDERR "Got:\n--8<---\n$page\n-->8---\n";
	if ($page =~ m/^HTTP\S+\s+302\b/) {
	    # HTTP/1.1 302 Moved Temporarily
	    # Date: Thu, 29 Sep 2005 07:23:42 GMT
	    # Server: Apache/1.3.31 (Unix) mod_ssl/2.8.17 OpenSSL/0.9.7d
	    # Location: http://forums1.itrc.hp.com/service/forums/publicProfile.do?userId=BR41967&admit=-682735245+1127978621707+28353475
	    my ($new_url) = ($page =~ m/^Location:\s*(\S+)/m);
	    $opt_v > 0 and print STDERR "$url returned a 302. Retrying on $new_url\n";
	    $opt_v > 3 and print STDERR "** $_\n" for split m/\n/, $page;
	    $new_url =~ s{^.*$ux_url}{/$ux_url}o;
	    return get_request ($new_url);
	    }
	}
    $page;
    } # get_request

sub webget ($;$)
{
    my ($p, $F) = @_;
    local $/;

    (my $url = $p) =~ s{^(?:http://)?(?:$server|$servip)}{}o;
    $url =~ s{^/?(?:$ux_url/)?}{};
    (my $f = $url) =~ s:.*/::;
    $F ||= $f;
    unlink $F;

    $opt_v     and printf STDERR "get %-38s\t", $F;
    $opt_v > 1 and printf STDERR "--- http://$server/$ux_url/$url\n";

    #print STDERR "$opt_v: get_request (/$ux_url/$url) => $F\n";
    my $page = get_request ("/$ux_url/$url");
    my $t = $page ? length ($page) : 0;
    if ($page) {
	undef $p;
	$t < 1024 and return 0; # Cannot be a real page
	$F eq "-" and return $page;
	open  $p, "> $F" or die "$F: $!\n";
	print $p $page;
	close $p;
	}
    $t;
    } # webget

1;
